var R = require ( 'ramda' );
var utils = require ( './utils' );

function stringifyNumber(n) {
    var special = ['zeroth','first', 'second', 'third', 'fourth', 'fifth', 'sixth', 'seventh', 'eighth', 'ninth', 'tenth', 'eleventh', 'twelfth', 'thirteenth', 'fourteenth', 'fifteenth', 'sixteenth', 'seventeenth', 'eighteenth', 'nineteenth'];
    var deca = ['twent', 'thirt', 'fourt', 'fift', 'sixt', 'sevent', 'eight', 'ninet'];
    if (n < 20) return special[n];
    if (n%10 === 0) return deca[Math.floor(n/10)-2] + 'ieth';
    return deca[Math.floor(n/10)-2] + 'y-' + special[n%10];
}

function initBookings() {
    return {
        total: 0,
        totalString: stringifyNumber(0),
        home: 0,
        homeString: stringifyNumber(0),
        away: 0,
        awayString: stringifyNumber(0)
    }
};

exports.particleType = 'rbb-bookings';

exports.assemblerType = 'rbb-bookings';

exports.initial = function ( config ) {
    var initPlayers = function ( side ) {
        return R.length ( R.reject ( function ( player ) {
            return player.position === 'Substitute';
        }, R.path ( [ 'players' ], R.find ( function ( team ) {
            return team.side === side;
        }, R.values ( config.teams ) ) ) || [] ) );
    };

    var homeTeamName = utils.chooseTeamName ( R.values ( config.teams ), 'Home' );
    var awayTeamName = utils.chooseTeamName ( R.values ( config.teams ), 'Away' );

    return {
        particle: {
            version: config.version,
            type: exports.particleType,
            assemblerType: exports.assemblerType,
            title: 'Booking: ' + [
                homeTeamName,
                awayTeamName
            ].join ( ' vs ' ),
            data: {
                referee: config.referee,
                venue: config.venue,
                teams: {
                    home: { rbbName: homeTeamName },
                    away: { rbbName: awayTeamName }
                },
                bookings: {
                    coachRed: initBookings(),
                    red: initBookings(),
                    yellow: initBookings(),
                    secondYellow: initBookings()
                }
            }
        },
        teams: config.teams,
        playersOnPitch: {
            home: initPlayers ( 'Home' ),
            away: initPlayers ( 'Away' )
        }
    };
};

exports.reducer = function ( context, event ) {
    var side = context.teams[event.team.id].side.toLowerCase();

    var findPlayer = function ( playerId ) {
        return R.find ( function ( player ) {
            return player.id === playerId;
        }, R.concat ( R.values ( context.teams )[0].players, R.values ( context.teams )[1].players ) );
    };

    var findOfficial = function ( officialId ) {
        return R.find ( function ( official ) {
            return official.id === officialId;
        }, R.concat ( R.values ( context.teams )[0].officials, R.values ( context.teams )[1].officials ) );
    };

    var player = event.player ? findPlayer ( event.player.id ) : undefined;
    var playerName = ( player && player.name ) ? ( player.name.known || player.name.first + ' ' + player.name.last ) : '';

    var official = event.qualifiers['coach-id'] ? findOfficial ( parseInt ( event.qualifiers['coach-id'].value, 10 ) ) : undefined;
    var officialName = ( official && official.name ) ? ( official.name.known || official.name.first + ' ' + official.name.last ) : '';

    var team = event.team ? R.omit ( [ 'players', 'officials' ], context.teams[event.team.id] ) : undefined;

    var playersOnPitchPath = [ 'playersOnPitch', R.toLower ( context.teams[event.team.id].side ) ];

    if ( event.type.name === 'Player Off' || event.type.name === 'Player retired' ) {
        return R.assocPath ( playersOnPitchPath, R.path ( playersOnPitchPath, context ) - 1, context );
    }

    if ( event.type.name === 'Player on' || event.type.name === 'Player returns' ) {
        return R.assocPath ( playersOnPitchPath, R.path ( playersOnPitchPath, context ) + 1, context );
    }

    if (event.type.name === 'Card') {
        if (R.has('second-yellow',event.qualifiers)) {
            context.particle.data.bookings.secondYellow.total++;
            context.particle.data.bookings.secondYellow.totalString = stringifyNumber(context.particle.data.bookings.secondYellow.total);
            context.particle.data.bookings.secondYellow[side]++;
            context.particle.data.bookings.secondYellow[side + 'String'] = stringifyNumber(context.particle.data.bookings.secondYellow[side]);
        } else if (R.has('red-card',event.qualifiers)) {
            if (R.has('coach-id',event.qualifiers)) {
                context.particle.data.bookings.coachRed.total++;
                context.particle.data.bookings.coachRed.totalString = stringifyNumber(context.particle.data.bookings.coachRed.total);
                context.particle.data.bookings.coachRed[side]++;
                context.particle.data.bookings.coachRed[side + 'String'] = stringifyNumber(context.particle.data.bookings.coachRed[side]);
            } else {
                context.particle.data.bookings.red.total++;
                context.particle.data.bookings.red.totalString = stringifyNumber(context.particle.data.bookings.red.total);
                context.particle.data.bookings.red[side]++;
                context.particle.data.bookings.red[side + 'String'] = stringifyNumber(context.particle.data.bookings.red[side]);
            }
        } else {
            context.particle.data.bookings.yellow.total++;
            context.particle.data.bookings.yellow.totalString = stringifyNumber(context.particle.data.bookings.yellow.total);
            context.particle.data.bookings.yellow[side]++;
            context.particle.data.bookings.yellow[side + 'String'] = stringifyNumber(context.particle.data.bookings.yellow[side]);
        }
    }

    if ( event.type.name === 'Card' ) {
        var redCard = R.has ( 'red-card', event.qualifiers ) || R.has ( 'second-yellow', event.qualifiers );

        var isCoach = R.has ( 'coach-id', event.qualifiers );

        var paths = [
            [ [ 'particle', 'data', 'booking', 'player' ], isCoach ? undefined : R.merge ( player, { name: playerName } ) ],
            [ [ 'particle', 'data', 'booking', 'coach' ], isCoach ? R.merge ( official, { name: officialName } ) : undefined ],
            [ [ 'particle', 'data', 'booking', 'team' ], R.merge ( team, {
                rbbName: utils.chooseTeamName ( R.values ( context.teams ), team.side )
            } ) ],
            [ [ 'particle', 'data', 'booking', 'red' ], redCard ],
            [ [ 'particle', 'data', 'booking', 'yellow' ], R.has ( 'yellow-card', event.qualifiers ) ],
            [ [ 'particle', 'data', 'booking', 'secondYellow' ], R.has ( 'second-yellow', event.qualifiers ) ],
            [ [ 'particle', 'data', 'booking', 'playersOnPitch' ], R.path ( playersOnPitchPath, context ) - ( ( redCard && ! isCoach ) ? 1 : 0 ) ],
            [ playersOnPitchPath, R.path ( playersOnPitchPath, context ) - ( ( redCard && ! isCoach ) ? 1 : 0 ) ]
        ];

        return R.reduce ( function ( context, path ) {
            return R.assocPath ( path[0], path[1], context );
        }, context, paths );
    }

    return context;
};
