var utils = require('./utils');
var R = require ( 'ramda' );

var assemblerType = 'rbb-combined-team-average-touch-position';

exports.particleType = 'rbb-double-pitch-map';

exports.initial = function (config) {

    var findTeam = utils.chooseTeamName (R.values(config.teams));
    
    return {
        particle: {
            title: '',
            type: exports.particleType,
            assemblerType: assemblerType,
            version: config.version,
            uniqueId: [ assemblerType, 0 ].join ( '-' ),
            data: {
                showDirectionOfAttack: true,
                "is-average-touch": true,
                home: {
                    title: findTeam('home'),
                    events: [],
                    vertices: []
                },
                away: {
                    title: findTeam('away'),
                    events: [],
                    vertices: []
                }/*,
                animated: {
                    '1-minute-interval': []
                }*/
            }
        },
        teams: config.teams,
        prevPlayer: null,
        players: {},
        vertices: {
            home: {},
            away: {}
        },
        meta: {
            type: 'average-touch',
            publishTags: [ 'reducer:rbb-team-average-touch-position' ],
            generatedTimeStamp: null,            
            eventTimeStamp: null            
        },
        gameTime: null
    };
};

exports.reducer = function (context, event) {
    utils.extendTeamData(context, event);
    utils.extendPlayerData(context, event);
    if (event.team) {
        var team = context.teams[event.team.id];
        var side = team.side.toLowerCase();
    }

    context.particle.title = 'Average touch positions (' + utils.getFormattedTime(event) + ')';
        
    if ( event.type.name === 'End' ) {
        if ( event.period_id === 1 ) {
            context.particle.title = 'Average touch positions (half time)';
        } else if ( event.period_id === 2 ) {
            context.particle.title = 'Average touch positions (full time)';
        }
    }

    if (event.type.name === 'Team set up') {
        team.involved.forEach(function (id, i) {
            var player = context.players[id];
	    player.total = {x: 0, y: 0};
	    player.average = {},
	    player.count = 0;
	    player.id = id;
        });
    } else if (event.type.name !== 'Pass') {
        return context;
    } else if (event.player && 'x' in event && 'y' in event) {
        var player = context.players[event.player.id];

        player.count++;
        player.total.x += Number(event.x);
        player.total.y += Number(event.y);
        player.average.x = player.total.x / player.count;
        player.average.y = player.total.y / player.count;

        var side = context.teams[player.team].side.toLowerCase();
	    var data = context.particle.data;
        var start = {
            x: utils.translateX('Left to Right', player.average.x),
            y: utils.translateY('Left to Right', player.average.y)
        };
        var end = {
            x: utils.translateX('Left to Right', player.average.x),
            y: utils.translateY('Left to Right', player.average.y)
        };
        var oldEvent = data[side].events.find(function (x) {
            return player.id === x.player.id;
        });
        data[side].events = data[side].events.filter(function (x) {
            return player.id !== x.player.id;
        });
        data[side].events.push({
            type: 'dribble',
            side: side,
            qualifiers: {},
            player: {
                id: player.id,
                name: player.name,
                jersey: player.jersey_number,
                substitute: player.position === '5' || player.substitute,
                display: true
            },
            start: start,
            end: end,
            weight: oldEvent ? (oldEvent.weight + 0.015) : 0.5
        });
        context.gameTime = utils.createGameTimeObject(event, context.gameTime);
        // opta recorded time
        //context.meta.eventTimeStamp = utils.getOptaEventTimeStamp(event);
        //context.particle.uniqueId = [ assemblerType, event.id ].join ( '-' );

        if ( context.prevEvent &&
            context.prevEvent.team &&
            event.team &&
            context.prevEvent.player &&
            event.player &&
            context.prevEvent.player.id !== event.player.id &&
            context.prevEvent.team.id === event.team.id ) {
            var oldVertex = context.vertices[side][[context.prevEvent.player.id, event.player.id].join ( '-' )];

            var vertex = {
                from: context.prevEvent.player.id,
                to: event.player.id,
                weight: oldVertex ? ( oldVertex.weight + 1 ) : 1
            };

            context.vertices[side][[context.prevEvent.player.id, event.player.id].join ( '-' )] = vertex;
        }

        context.prevEvent = event;

        data[side].vertices = R.values(context.vertices[side]).map(function(pass){
            var startEvent = data[side].events.find(function(event){
                return Number(event.player.id) === Number(pass.from);
            });
            var endEvent = data[side].events.find(function(event){
                return Number(event.player.id) === Number(pass.to);
            });
            return {
                type: 'vertex',
                side: side,
                weight: pass.weight,
                player: {
                    id: pass.from,
                    name: '',
                    jersey: '',
                    display: false
                },
                start: startEvent.start,
                end: endEvent.start
            };
        });

        /*if (data.animated['1-minute-interval'].length > event.min) {
            data.animated['1-minute-interval'][event.min] = R.clone(R.pick(['home','away'], data));
        } else {
            while (data.animated['1-minute-interval'].length < event.min) {
                data.animated['1-minute-interval'].push(R.clone(R.pick(['home','away'], data)));
            }
        }*/
    }

    return context;
};
