var debug = false;

var R = require ( 'ramda' );
var utils = require ( './utils' );

var log = debug ? console.log : function () {};

var periods = {
    '1': 'First half',
    '2': 'Second half',
    '3': 'First half (extra time)',
    '4': 'Second half (extra time)',
    '5': 'Penalty shoot-out',
    '14': 'Post game',
    '15': 'Pre-game',
    '16': 'Pre-match'
};

var otherSide = function (side) {
    return side === 'home' ? 'away' : 'home';
};

exports.particleType = 'rbb-fixture';

exports.assemblerType = 'rbb-fixture';

exports.initial = function ( config ) {
    var findTeam = function ( teams, side ) {
        return R.find ( function ( team ) {
            return team.side === side;
        }, teams );
    };

    return {
        particle: {
            version: config.version,
            type: exports.particleType,
            assemblerType: exports.assemblerType,
            title: 'Fixture particle: ' + utils.chooseTeamName ( R.values ( config.teams ), 'Home' ) + ' vs ' + utils.chooseTeamName ( R.values ( config.teams ), 'Away' ),
            data: {
                teams: {
                    home: R.merge ( R.omit ( [ 'players' ], findTeam ( R.values ( config.teams ), 'Home' ) ), {
                        rbbName: utils.chooseTeamName ( R.values ( config.teams ), 'Home' ),
                        score: 0
                    } ),
                    away: R.merge ( R.omit ( [ 'players' ], findTeam ( R.values ( config.teams ), 'Away' ) ), {
                        rbbName: utils.chooseTeamName ( R.values ( config.teams ), 'Away' ),
                        score: 0
                    } )
                },
                venue: config.venue
            }
        },
        scoreHistory: [],
        teams: config.teams,
        events: []
    };
};

exports.reducer = function ( context, event ) {
	var side = context.teams[event.team.id].side.toLowerCase();
	var other = otherSide(side);

    var sideScorePath = [ 'particle', 'data', 'teams', side, 'score' ];
    var otherScorePath = [ 'particle', 'data', 'teams', other, 'score' ];

    var sideGoal = event.type.name === 'Goal' && ! R.has ( 'own-goal', event.qualifiers );
    var otherGoal = event.type.name === 'Goal' && R.has ( 'own-goal', event.qualifiers );

    var addedAttributes = [
        [ [ 'particle', 'data', 'period' ], ( periods[event.period_id] || 'Unknown' ) ],
        [ [ 'particle', 'data', 'period_lowercase' ], R.toLower ( ( periods[event.period_id] || 'unknown' ) ) ],
        [ sideScorePath, R.path ( sideScorePath, context ) + ( sideGoal ? 1 : 0 ) ],
        [ otherScorePath, R.path ( otherScorePath, context ) + ( otherGoal ? 1 : 0 ) ]
    ];

    if ( event.type.name === 'Condition change' ) {
        return context;
    }

    return R.reduce ( function ( context, attributeSpec ) {
        return R.assocPath ( attributeSpec[0], attributeSpec[1], context );
    }, context, addedAttributes );
};
