var utils = require('./utils');

var assemblerType = 'rbb-goals-and-assists';

exports.particleType = 'rbb-single-pitch-map';

exports.initial = function (config) {
    return {
        particle: {
            title: 'Goals and assists',
            type: exports.particleType,
            assemblerType: assemblerType,
            version: config.version,
            uniqueId: [ assemblerType, 0 ].join ( '-' ),
            data: {
                showDirectionOfAttack: true,
                events: [],
                goalkeepers: []
            },
        },
        teams: config.teams,
        players: {},
        meta: {
            type: 'goal',
            publishTags: [ 'reducer:rbb-goals-and-assists' ],
            generatedTimeStamp: null,
            eventTimeStamp: null           
        },
        gameTime: null,
        eventTime: null,
        assist: null,
        assists: []
    };
};

exports.reducer = function (context, event) {
    utils.extendTeamData(context, event);
    utils.extendPlayerData(context, event);

    if (event.type.name === 'Start') {
        context.assist = null;
    }
    else if (event.type.name === 'Pass' && event.qualifiers['assist']) {
        context.assists.push(event);
    }
    else if (event.type.name === 'Goal') {
        var time = utils.getFormattedTime(event);
        context.particle.title = (event.qualifiers['own-goal'] ? 'Own goal: ': 'Goal: ') +
            event.team.name + ' (' + time + ')';
        var events = context.particle.data.events = [];
        context.particle.data.goalkeepers = [];
	var team = context.teams[event.team.id];
	var direction = team.direction;
        var dribbled = false;
        var player, jersey, side;

        context.eventTime = utils.createGameTimeObject( event );

        context.particle.data['attack-left'] = (direction === 'Right to Left');
        context.particle.data['action-left'] = (direction === 'Right to Left');
        if (event.qualifiers['own-goal']) {
            // action is happening at opposite goal line
            context.particle.data['action-left'] = !(context.particle.data['action-left']);
        }
        if ( event.qualifiers['related-event-id'] ) {
            context.assist = context.assists.find(function(assist){
                return parseInt(event.qualifiers['related-event-id'].value, 10) === assist.event_id;
            }) || null;
        }
        context.assists = [];
        if (context.assist) {
            player = context.players[context.assist.player.id];
            jersey =  player ? player.jersey_number : '';
            direction = context.teams[context.assist.team.id].direction;
            var assist_end_x = context.assist.qualifiers['pass-end-x'].value;
            var assist_end_y = context.assist.qualifiers['pass-end-y'].value;
            events.push({
                type: "pass",
                side: context.teams[context.assist.team.id].side.toLowerCase(),
                "start": {
                    "x": utils.translateX(direction, context.assist.x),
                    "y": utils.translateY(direction, context.assist.y)
                },
                "end": {
                    "x": utils.translateX(direction, assist_end_x),
                    "y": utils.translateY(direction, assist_end_y)
                },
                "player": {
                    "jersey": jersey,
                    "display": true
                }
            });
            // calculate dribble
            if (event.x !== context.assist.x || event.y !== context.assist.y) {
                player = context.players[event.player.id];
                jersey =  player ? player.jersey_number : '';
                direction = context.teams[event.team.id].direction;
                dribbled = true;
                events.push({
                    type: 'dribble',
                    side: context.teams[event.team.id].side.toLowerCase(),
                    "start": {
                        "x": utils.translateX(direction, assist_end_x),
                        "y": utils.translateY(direction, assist_end_y)
                    },
                    "end": {
                        "x": utils.translateX(direction, event.x),
                        "y": utils.translateY(direction, event.y)
                    },
                    "player": {
                        "jersey": jersey,
                        "display": true
                    }
                });
            }
            context.assist = null;
        }
        player = context.players[event.player.id];
        jersey =  player ? player.jersey_number : '';
        direction = context.teams[event.team.id].direction;
        side = context.teams[event.team.id].side.toLowerCase();
        events.push({
            type: "goal",
            side: side,
            "start": {
                "x": utils.translateX(direction, event.x),
                "y": utils.translateY(direction, event.y)
            },
            "end": {
                "x": utils.translateX(direction, event.qualifiers['own-goal'] ? 0 : 100),
                "y": utils.translateY(direction, event.qualifiers['goal-mouth-y-co-ordinate'].value)
            },
            "player": {
                "jersey": jersey,
                "display": !dribbled
            }
        });
        // in an own-goal it's not clear which GK we have co-ordinates for, or
        // how they should be translated, so leave the GK off the pitch map
        if (!event.qualifiers['own-goal']) {
            context.particle.data.goalkeepers.push({
                "type": "position",
                "static": true,
                "side": utils.getOpposingSide(side),
                "start": {
                    "x": utils.translateX('Left to Right', event.qualifiers['gk-x-coordinate'].value),
                    "y": utils.translateY('Left to Right', event.qualifiers['gk-y-coordinate'].value)
                },
                "player": {
                    "jersey": "GK",
                    "display": true
                }
            });
        }
        context.meta.generatedTimeStamp = new Date().getTime();
        // opta recorded time
        context.meta.eventTimeStamp = utils.getOptaEventTimeStamp(event);
        context.particle.uniqueId = [ assemblerType, event.id ].join ( '-' );
    }
    context.gameTime = utils.createGameTimeObject(event, context.gameTime);
    return context;
};
