var utils = require('./utils');
var R = require('ramda');

// reference for goal event timestamps... won't be necessary for auto publish
var timestamps = {};

var assemblerType = 'rbb-goals-and-passes';

exports.particleType = 'rbb-single-pitch-map';

exports.initial = function (config) {
    var findTeam = utils.chooseTeamName (R.values(config.teams));

    return {
        particle: {
            type: exports.particleType,
            assemblerType: assemblerType,
            version: config.version,
            title: 'Goals and passes',
            uniqueId: [ assemblerType, 0 ].join ( '-' ),
            data: {
                animated: config.animated,
                showDirectionOfAttack: true,
                events: [],
                'percentage-touches-in-attack-half': 0,
                'current-period': {
                    id: -1,
                    'is-first-event': false,
                    'is-start-half-time': false
                },
                ownGoal: false,
                homeTeamName: findTeam ( 'home' ),
                awayTeamName: findTeam ( 'away' ),
                homeScore: 0,
                awayScore: 0,
                goalScorerPlayerName: '',
                goalScorerTeamName: '',
                otherTeamName: '',
                isEarly: false,
                isLate: false,
                isLateLate: false,
                isJustBeforeHalftime: false,
                isEqualiser: false,
                isOpener: false,
                is20Lead: false,
                is30Lead: false,
                is60Lead: false,
                is70Lead: false,
                is1GoalLead: false,
                is2GoalLead: false,
                is2nd1GoalLead: false,
                is2nd2GoalLead: false,
                isAnschlussTreffer: false,
                isGoalPulledBack: false,
                isConsolationGoal: false,
                isHitForSix: false
            }
        },
        scoreHistory: [],
        interrupted: false,
        events: [],
        teams: config.teams || {},
        players: {},
        scores: {
            home: 0,
            away: 0
        },
        isHalf: config.isHalf,
        meta: {
            type: 'goal',
            publishTags: [ 'reducer:rbb-goals-and-passes' ],
            generatedTimeStamp: null,  
            eventTimeStamp: null          
        },
        gameTime: null,
        eventTime: null
    };
};

function resetPassChain(context) {
    context.interrupted = false;
    context.events = [];
    delete context.last_x;
    delete context.last_y;
}

function isDribbleInAttackingHalf(context, event) {
    return Math.max(context.last_x, event.x) >= 50;
}

exports.reducer = function (context, event) {
    utils.extendTeamData(context, event);
    utils.extendPlayerData(context, event);

    var team = event.team && context.teams[event.team.id];
    var player = event.player && context.players[event.player.id];
    
    if (event.type.name === 'Start') {
        resetPassChain(context);
    }
    else if (event.type.name === 'Goal') {
        var side = context.teams[event.team.id].side.toLowerCase();
        var otherSide = ( side === 'home' ? 'away' : 'home' );
        var time = utils.getFormattedTime(event);

        var previous1GoalLeads = R.filter ( function ( scores ) {
            return ( scores[side] - scores[otherSide] ) === 1;
        }, context.scoreHistory ).length;

        var previous2GoalLeads = R.filter ( function ( scores ) {
            return ( scores[side] - scores[otherSide] ) === 2;
        }, context.scoreHistory ).length;

        // set up the current period indicators for rules engine
        context.particle.data['current-period'] = utils.setTimePeriodForRules(event, context);

        context.particle.data.goalScorerPlayerName = player.name;
        context.particle.data.goalScorerTeamName = side === 'home' ? context.particle.data.homeTeamName : context.particle.data.awayTeamName;
        context.particle.data.otherTeamName = otherSide === 'home' ? context.particle.data.homeTeamName : context.particle.data.awayTeamName;

        if ( event.qualifiers['own-goal'] ) {
            context.scores[otherSide]++;
            context.particle.data.ownGoal = true;
        } else {
            context.scores[side]++;
            context.particle.data.ownGoal = false;
        }

        context.particle.data.homeScore = context.scores.home;
        context.particle.data.awayScore = context.scores.away;

        context.particle.data.isEqualiser = ( context.scores.home === context.scores.away );

        context.particle.data.isOpener = ( context.scores[side] === 1 ) && ( context.scores[otherSide] === 0 );
        context.particle.data.is20Lead = ( ( context.scores[side] === 2 ) && ( context.scores[otherSide] === 0 ) );
        context.particle.data.is30Lead = ( ( context.scores[side] === 3 ) && ( context.scores[otherSide] === 0 ) );
        context.particle.data.is60Lead = ( ( context.scores[side] === 6 ) && ( context.scores[otherSide] === 0 ) );
        context.particle.data.is70Lead = ( ( context.scores[side] === 7 ) && ( context.scores[otherSide] === 0 ) );
        context.particle.data.is1GoalLead = ( ( context.scores[side] - context.scores[otherSide] ) === 1 );
        context.particle.data.is2GoalLead = ( ( context.scores[side] - context.scores[otherSide] ) === 2 );

        context.particle.data.is2nd1GoalLead = ( context.particle.data.is1GoalLead && ( previous1GoalLeads === 1 ) );
        context.particle.data.is2nd2GoalLead = ( context.particle.data.is2GoalLead && ( previous2GoalLeads === 1 ) );

        context.particle.data.isEarly = ( event.min < 15 );
        context.particle.data.isLate = ( event.min > 45 );
        context.particle.data.isLateLate = ( event.min > 80 );
        
        context.particle.data.isJustBeforeHalftime = ( event.min > 42 ) && ( context.particle.data['current-period'].id === 1 );

        context.particle.data.isAnschlussTreffer = ( ( context.scores[side] === 1 ) && ( context.scores[otherSide] === 2 ) && ( event.min < 75 ) );
        context.particle.data.isGoalPulledBack = ( ( context.scores[otherSide] - context.scores[side] ) === 2 );
        context.particle.data.isConsolationGoal = ( ( ( context.scores[otherSide] - context.scores[side] ) > 2 ) && ( event.min > 75 ) );

        context.particle.data.isHitForSix = context.particle.data.is60Lead;

        if (utils.isPassChainStart(event, context.last_team) || context.interrupted) {
            resetPassChain(context);
        }
        // context.title = event.team.name + (
        //     event.qualifiers['own-goal'] ? ' own goal: ': ' goal'
        // );

        context.particle.title = [
            //(event.qualifiers['own-goal'] ? 'Own goal!:' : 'Goal!:'),
            [
                [ context.particle.data.homeTeamName, context.scores.home ].join ( ' ' ),
                [ context.scores.away, context.particle.data.awayTeamName ].join ( ' ' )
            ].join ( ' - ' ),
            [ '(', player.name, ', ', time, ')' ].join ( '' )
        ].join ( ' ' );

        team = context.teams[event.team.id];
        context.particle.data.goalkeepers = [];

        context.eventTime = utils.createGameTimeObject( event );

        jersey =  player ? player.jersey_number : '';
        direction = context.teams[event.team.id].direction;

        context.particle.data['attack-left'] = (direction === 'Right to Left');
        context.particle.data['action-left'] = (direction === 'Right to Left');
        if (event.qualifiers['own-goal']) {
            // action is happening at opposite goal line
            context.particle.data['action-left'] = !(context.particle.data['action-left']);
        }

        // calculate dribble event
        dribbled = false;
        if (context.last_x !== undefined && context.last_y !== undefined) {
            if (context.last_x !== event.x || context.last_y !== event.y) {
                dribbled = true;
                
                context.events.push({
                    type: "dribble",
                    side: side,
                    qualifiers: {},
                    player: {
                        name: player.name,
                        jersey: jersey,
                        display: true
                    },
                    start: {
                        x: utils.translateX(direction, context.last_x),
                        y: utils.translateY(direction, context.last_y)
                    },
                    end: {
                        x: utils.translateX(direction, event.x),
                        y: utils.translateY(direction, event.y)
                    },
                    "is-in-attacking-half": isDribbleInAttackingHalf(context, event)                    
                });
            }
        }

        context.events.push({
            type: "goal",
            side: side,
            "own-goal": (event.qualifiers['own-goal']) ? true : false,
            "start": {
                "x": utils.translateX(direction, event.x),
                "y": utils.translateY(direction, event.y)
            },
            "end": {
                "x": utils.translateX(direction, event.qualifiers['own-goal'] ? 0 : 100),
                "y": utils.translateY(direction, event.qualifiers['goal-mouth-y-co-ordinate'].value)
            },
            "player": {
                name: player.name,
                "jersey": jersey,
                "display": !dribbled
            },
            "is-in-attacking-half": utils.isInAttackingHalf(event) 
        });
        // in an own-goal it's not clear which GK we have co-ordinates for, or
        // how they should be translated, so leave the GK off the pitch map
        if (!event.qualifiers['own-goal']) {
            context.particle.data.goalkeepers.push({
                "type": "position",
                "static": true,
                "side": utils.getOpposingSide(side),
                "start": {
                    "x": utils.translateX('Left to Right', event.qualifiers['gk-x-coordinate'].value),
                    "y": utils.translateY('Left to Right', event.qualifiers['gk-y-coordinate'].value)
                },
                "player": {
                    name: player.name,
                    "jersey": "GK",
                    "display": true
                }
            });
        }

        context.particle.data.events = context.events.slice();

        // TODO: evaluate if we still need to do this once auto-choosing a template
        // quick remove own half when only showing half pitch
        // reason: animated graphic, don't want to have long delay for items off screen
        if(context.isHalf) {
            context.particle.data.events = R.filter(function(event) {
                if( event['own-goal'] ) {
                    return true;
                } else if(context.particle.data['attack-left']) {
                    return event.start.x < 250 || event.end.x < 250;                    
                } else {
                    return event.start.x > 250 || event.end.x > 250; 
                }
            }, context.particle.data.events);
        }
        // end own half remove
        // 
        
        // keeping reference to a timestamp for the event so as not to lose it when new events come in
        if(!timestamps[event.event_id]) {
            timestamps[event.event_id] = new Date().getTime();
        }
        context.meta.generatedTimeStamp = timestamps[event.event_id];            

        // opta recorded time
        context.meta.eventTimeStamp = utils.getOptaEventTimeStamp(event);

        // time within the match
        context.gameTime = context.eventTime;

        context.particle.data.animated = context.particle.data.events.length > 2;

        // calculate percentage in attacking half for rules to choose templates
        var inAttack = R.filter( R.propEq('is-in-attacking-half', true), context.particle.data.events);
        context.particle.data['percentage-touches-in-attack-half'] = inAttack.length/context.particle.data.events.length;

        context.particle.uniqueId = [ assemblerType, event.id ].join ( '-' );

        context.scoreHistory.push ( R.clone ( context.scores ) );
    }
    else if (utils.isGenericPassEvent(event)) {
        if (utils.isPassChainStart(event, context.last_team) || context.interrupted) {
            resetPassChain(context);
        }
        team = context.teams[event.team.id];
        var team_name = team ? team.name : 'Unknown team';
        var end_x = event.qualifiers['pass-end-x'].value;
        var end_y = event.qualifiers['pass-end-y'].value;
        var jersey =  player ? player.jersey_number : '';
        var direction = context.teams[event.team.id].direction;
        // calculate dribble event
        var dribbled = false;
        if (context.last_x !== undefined && context.last_y !== undefined) {
            if (context.last_x !== event.x || context.last_y !== event.y) {
                dribbled = true;
                context.events.push({
                    type: 'dribble',
                    side: context.teams[event.team.id].side.toLowerCase(),
                    qualifiers: {},
                    player: {
                        name: player.name,
                        jersey: jersey,
                        display: true
                    },
                    start: {
                        x: utils.translateX(direction, context.last_x),
                        y: utils.translateY(direction, context.last_y)
                    },
                    end: {
                        x: utils.translateX(direction, event.x),
                        y: utils.translateY(direction, event.y)
                    },
                    "is-in-attacking-half": isDribbleInAttackingHalf(context, event) 
                });
            }
        }
        var qualifiers = {};
        for (var k in event.qualifiers) {
            var q = event.qualifiers[k];
            if (q.type.values === '') {
                qualifiers[k] = true;
            }
        }
        context.events.push({
            type: event.qualifiers['throw-in'] ? 'throwin' : 'pass',
            side: context.teams[event.team.id].side.toLowerCase(),
            qualifiers: qualifiers,
            player: {
                name: player.name,
                jersey: jersey,
                display: !dribbled
            },
            start: {
                x: utils.translateX(direction, event.x),
                y: utils.translateY(direction, event.y)
            },
            end: {
                x: utils.translateX(direction, end_x),
                y: utils.translateY(direction, end_y)
            },
            "is-in-attacking-half": utils.isInAttackingHalf(event) 
        });
        context.last_x = end_x;
        context.last_y = end_y;
        context.last_team = event.team.id;

        context.gameTime = utils.createGameTimeObject(event, context.gameTime);
    }
    else if (utils.isPassChainEnd(event, context.last_team)) {
        context.interrupted = true;
    }
    return context;
};
