var utils = require ( './utils' );
var R = require('ramda');

exports.initial = function (config) {
    return {
        particle: {
	    publishes: {
		byName: {},
		byType: {}
	    },
            match: {
                playedMinutes: 0,
                playedSeconds: 0,
                currentPeriod: 0,
                tenMinutes: 0,
		        fifteenMinutes: 0,
                startPeriod: 0,
                endPeriod: 0,
                penaltiesFollow: false,
                extraTimeFollow: false,
                passes: {
                    total: 0,
                    home: 0,
                    away: 0
                },
                passesPerShot: {
                    total: 0,
                    home: 0,
                    away: 0
                },
		        shots: initShots(),
                touches: {
                    total: 0,
                    home: 0,
                    away: 0,
                    distance: 0,
                    ahead: 'none',
                    thirds: {
                        home: initTouchThirds (),
                        away: initTouchThirds (),
                        midfieldDistance: 0
                    }
                },
                bookings: {
                    coachRed: initBookings(),
                    red: initBookings(),
                    yellow: initBookings(),
                    secondYellow: initBookings()
                }
            },
            home: {
                score: 0,
                penalties: 0,
                previous_score: 0,
                previous_penalties: 0,
                scorer: false,
                previous_scorer: false
            },
            away: {
                score: 0,
                penalties: 0,
                previous_score: 0,
                previous_penalties: 0,
                scorer: false,
                previous_scorer: false
            },
            scoreDifference: 0,
            previousScoreDifference: 0,
            penaltyDifference: 0,
            previousPenaltyDifference: 0,
            penaltyWin: false,
	        period: initPeriod(null),
            eventType: null,
            fixtureType: config.fixtureType
        },
        teams: config.teams,
        interrupted: false,
        possession_secs_10min: {
            home: 0,
            away: 0
        }
    };
};

function initTouchThirds () {
    return {
        defence: {
            total: 0,
            percentage: 33
        },
        midfield: {
            total: 0,
            percentage: 33
        },
        attack: {
            total: 0,
            percentage: 33
        }
    };
}

function isGamePeriodEvent(event) {
    return event.period_id > 0 && event.period_id < 6;
}

function otherSide(side) {
    return side === 'home' ? 'away' : 'home';
}

function initPeriod(periodId) {
    return {
	    id: periodId,
	    shots: initShots()
    };
}

function initShots() {
    return {
        total: 0,
        onTarget: 0,
        onTargetRatio: 0,
        home: {
            total: 0,
            onTarget: 0,
            onTargetRatio: 0
        },
        away: {
            total: 0,
            onTarget: 0,
            onTargetRatio: 0
        }
    };
}

function initBookings() {
    return {
        total: 0,
        home: 0,
        away: 0
    };
};

exports.reducer = function (context, event) {
    var publishes;

    if (event.type.name === 'Rule set up') {
        if ( event.rule ) {
            publishes = context.particle.publishes;
            publishes.byType[event.rule.type] = {total: 0};
            publishes.byName[event.rule.name] = {total: 0};
        }
        return context;
    } else if (event.type.name === 'Publish') {
        if ( event.rule ) {
            publishes = context.particle.publishes;

            if ( publishes.byType[event.rule.type] ) {
                publishes.byType[event.rule.type].total++;
                publishes.byType[event.rule.type].currentPeriod++;
            } else {
                publishes.byType[event.rule.type] = {
		    total: 1,
		    currentPeriod: 1
		};
            }

            if ( publishes.byName[event.rule.name] ) {
                publishes.byName[event.rule.name].total++;
                publishes.byName[event.rule.name].currentPeriod++;
            } else {
                publishes.byName[event.rule.name] = {
		    total: 1,
		    currentPeriod: 1
		};
            }
        }
        return context;
    }
    
    var side = context.teams[event.team.id].side.toLowerCase();

    context.particle.eventType = event.type.name;

    if ( utils.isTouchEvent ( event ) ) {
        var touches = context.particle.match.touches;
        touches.total++;
        touches[side]++;
        touches.distance = Math.abs ( touches.home - touches.away );
        touches.ahead = ( touches.home > touches.away ? 'home' : ( touches.home < touches.away ? 'away' : 'none' ) );
        touches.thirds[side] = utils.transformTouchThirds ( event, touches.thirds[side] );
        touches.thirds.midfieldDistance = Math.abs ( touches.thirds.home.midfield.percentage - touches.thirds.away.midfield.percentage );
    }

    context.particle.isOwnGoal = event.type.name === 'Goal' && typeof event.qualifiers['own-goal'] !== 'undefined';

    if (event.period_id !== context.particle.period.id) {
	    context.particle.period = initPeriod(event.period_id);
    }
    // TODO: remove this once no rules rely upon match.curentPeriod
    if ( typeof event.period_id !== 'undefined' ) {
        context.particle.match.currentPeriod = event.period_id;
    }
    
    if (isGamePeriodEvent(event)) {
        context.particle.match.playedMinutes = event.min;
        context.particle.match.playedSeconds = event.sec;
        context.particle.match.tenMinutes = Math.floor(event.min / 10);
        context.particle.match.fifteenMinutes = Math.floor(event.min / 15);
    }
    if ( event.type.name === 'End' ) {
        context.particle.match.endPeriod = event.period_id;
        context.particle.match.penaltiesFollow = R.has ( 'goes-to-penalties', event.qualifiers );
        context.particle.match.extraTimeFollow = R.has ( 'goes-to-extra-time', event.qualifiers );
        return context;
    }
    if (event.type.name === 'Goal') {
        side = context.teams[event.team.id].side.toLowerCase();
        var other = otherSide(side);
        if (event.qualifiers['own-goal']) {
            side = otherSide(side);
            other = otherSide(other);
        }
        if (context.particle[side].scorer) {
            context.particle[side].previous_scorer = true;
            context.particle[other].previous_scorer = false;
        }
        else if (context.particle[other].scorer) {
            context.particle[other].previous_scorer = true;
            context.particle[side].previous_scorer = false;
        }
        context.particle[other].scorer = false;
        context.particle[side].scorer = true;
        if (event.period_id === 5) {
            context.particle[side].previous_penalties = context.particle[side].penalties;
            context.particle[other].previous_penalties = context.particle[other].penalties;
            context.particle[side].penalties++;
            context.particle.previousPenaltyDifference = context.particle.penaltyDifference;
            context.particle.penaltyDifference =
                Math.abs(context.particle.home.penalties - context.particle.away.penalties);
        } else {
            context.particle[side].previous_score = context.particle[side].score;
            context.particle[other].previous_score = context.particle[other].score;
            context.particle[side].score++;
            context.particle.previousScoreDifference = context.particle.scoreDifference;
            context.particle.scoreDifference =
                Math.abs(context.particle.home.score - context.particle.away.score);
        }
    }

    if (event.type.name === 'Start') {
        context.particle.match.startPeriod = event.period_id;
	publishes = context.particle.publishes;
	for (var name in publishes.byName) {
	    publishes.byName[name].currentPeriod = 0;
	}
	for (var type in publishes.byType) {
	    publishes.byType[type].currentPeriod = 0;
	}
        return context;
    }

    if (event.type.name === 'Card') {
        if (R.has('second-yellow',event.qualifiers)) {
            context.particle.match.bookings.secondYellow.total++;
            context.particle.match.bookings.secondYellow[side]++;
            return context;
        }

        if (R.has('red-card',event.qualifiers)) {
            if (R.has('coach-id',event.qualifiers)) {
                context.particle.match.bookings.coachRed.total++;
                context.particle.match.bookings.coachRed[side]++;
                return context;
            }
            context.particle.match.bookings.red.total++;
            context.particle.match.bookings.red[side]++;
            return context;
        }
        context.particle.match.bookings.yellow.total++;
        context.particle.match.bookings.yellow[side]++;
        return context;
    }

    // cumulative shots calculations
    if (utils.isShot(event) && !R.has('own-goal', event.qualifiers)) {
        var match = context.particle.match;
        var period = context.particle.period;
        period.shots.total++;
        match.shots.total++;
        period.shots[side].total++;
        match.shots[side].total++;
    	if (utils.isShotOnTarget(event)) {
            period.shots.onTarget++;
            match.shots.onTarget++;
            period.shots[side].onTarget++;
            match.shots[side].onTarget++;
	    }
        match.shots.lastUpdated = side;
        period.shots.onTargetRatio =
            period.shots.onTarget / period.shots.total;
        match.shots.onTargetRatio =
            match.shots.onTarget / match.shots.total;
        period.shots[side].onTargetRatio =
            period.shots[side].onTarget / period.shots[side].total;
        match.shots[side].onTargetRatio =
            match.shots[side].onTarget / match.shots[side].total;

        if (period.id === 5) {
            if (period.shots.total < 10) {
                if (5 - period.shots.away.total < context.particle.home.penalties) {
                    context.particle.penaltyWin = true;
                } else if (5 - period.shots.home.total < context.particle.away.penalties) {
                    context.particle.penaltyWin = true;
                }
            } else {
                if (period.shots.total%2 === 0) {
                    if (context.particle.home.penalties !== context.particle.away.penalties) {
                        context.particle.penaltyWin = true;
                    }
                }
            }
        }
    }

    // cumulative passes & effectiveness

    if ( utils.isPass ( event ) ) {
        context.particle.match.passes.total++;
        context.particle.match.passes[side]++;
        context.particle.match.passesPerShot.total = context.particle.match.passes.total / ( context.particle.match.shots.onTarget || 1 );
        context.particle.match.passesPerShot[side] = context.particle.match.passes[side] / ( context.particle.match.shots[side].onTarget || 1 );
    }

    return context;
};
