var utils = require('./utils');

var assemblerType = 'rbb-passes-on-pitch';

exports.particleType = 'rbb-single-pitch-map';

exports.initial = function (config) {
    return {
        particle: {
            title: 'Passes',
            type: exports.particleType,
            assemblerType: assemblerType,
            version: config.version,
            uniqueId: [ assemblerType, 0 ].join ( '-' ),
            data: {
                animated: false,
                showDirectionOfAttack: true
            }
        },
        interrupted: false,
        events: [],        
        teams: config.teams,
        players: {},
        meta: {
            type: 'passes',
            publishTags: [ 'reducer:rbb-passes-on-pitch' ],
            generatedTimeStamp: null,
            eventTimeStamp: null            
        },
        gameTime: null
    };
};

function resetPassChain(context) {
    context.interrupted = false;
    context.events = [];
    context.particle.title = 'Passes';
    delete context.last_x;
    delete context.last_y;

    // TEMP: putting it inside the data attribute
    context.particle.data.events = [];
}

exports.reducer = function (context, event) {
    utils.extendTeamData(context, event);
    utils.extendPlayerData(context, event);

    if ( typeof window !== 'undefined' ) {
        if (!window.passes_on_pitch) {
            window.passes_on_pitch = 0;
        }
        window.passes_on_pitch++;
    }
    if (event.type.name === 'Start') {
        resetPassChain(context);
    }
    else if (utils.isGenericPassEvent(event)) {
        if (utils.isPassChainStart(event, context.last_team) || context.interrupted) {
            resetPassChain(context);
        }
        var team = context.teams[event.team.id];
        var team_name = team ? team.name : 'Unknown team';
        context.particle.title = event.team.name + ' passes';
        var end_x = event.qualifiers['pass-end-x'].value;
        var end_y = event.qualifiers['pass-end-y'].value;
        var player = context.players[event.player.id];
        var jersey =  player ? player.jersey_number : '';
        var direction = context.teams[event.team.id].direction;
        
        context.particle.data['attack-left'] = (direction === 'Right to Left');
        context.particle.data['action-left'] = (direction === 'Right to Left');

        // calculate dribble event
        var dribbled = false;
        if (context.last_x !== undefined && context.last_y !== undefined) {
            if (context.last_x !== event.x || context.last_y !== event.y) {
                dribbled = true;
                context.events.push({
                    type: 'dribble',
                    side: context.teams[event.team.id].side.toLowerCase(),
                    qualifiers: {},
                    player: {
                        jersey: jersey,
                        display: true
                    },
                    start: {
                        x: utils.translateX(direction, context.last_x),
                        y: utils.translateY(direction, context.last_y)
                    },
                    end: {
                        x: utils.translateX(direction, event.x),
                        y: utils.translateY(direction, event.y)
                    }
                });
            }
        }
        var qualifiers = {};
        for (var k in event.qualifiers) {
            var q = event.qualifiers[k];
            if (q.type.values === '') {
                qualifiers[k] = true;
            }
        }
        context.events.push({
            type: event.qualifiers['throw-in'] ? 'throwin' : 'pass',
            side: context.teams[event.team.id].side.toLowerCase(),
            qualifiers: qualifiers,
            player: {
                jersey: jersey,
                display: !dribbled
            },
            start: {
                x: utils.translateX(direction, event.x),
                y: utils.translateY(direction, event.y)
            },
            end: {
                x: utils.translateX(direction, end_x),
                y: utils.translateY(direction, end_y)
            }
        });
        context.last_x = end_x;
        context.last_y = end_y;
        context.last_team = event.team.id;

        // TEMP: putting it inside the data attribute
        context.particle.data.events = context.events;     

        context.particle.data.animated = context.particle.data.events.length > 3;       

        context.particle.uniqueId = [ assemblerType, event.id ].join ( '-' );
    }
    else if (utils.isPassChainEnd(event, context.last_team)) {
        context.interrupted = true;
    }


    context.gameTime = utils.createGameTimeObject(event, context.gameTime);

    // opta recorded time
    context.meta.eventTimeStamp = utils.getOptaEventTimeStamp(event);

        
    return context;
};
