var R = require ( 'ramda' );
var utils = require ( './utils' );


exports.particleType = 'rbb-passes';
exports.assemblerType = 'rbb-passes';


exports.initial = function ( config ) {

    return {
        particle: {
            version: config.version,
            type: exports.particleType,
            assemblerType: exports.assemblerType,
            title: utils.chooseTeamName ( R.values ( config.teams ), 'Home' ) + ' vs ' + utils.chooseTeamName ( R.values ( config.teams ), 'Away' ),
            data: {
                home: {
                    name: utils.chooseTeamName ( R.values ( config.teams ), 'Home' ),
                    events: [],
                    successful: 0,
                    unsuccessful: 0,
                    long: 0,
                    through: 0,
                    offensive: 0,
                    defensive: 0,
                    sideways: 0
                },
                away: {
                    name: utils.chooseTeamName ( R.values ( config.teams ), 'Away' ),
                    events: [],
                    successful: 0,
                    unsuccessful: 0,
                    long: 0,
                    through: 0,
                    offensive: 0,
                    defensive: 0,
                    sideways: 0
                }
            }
        },
        teams: config.teams
    };
};

exports.reducer = function ( context, event ) {

    // not a pass
    if( R.not( R.propEq('type_id', 1)(event) ) ) {
        return context;
    }

    var qualifierIds = R.pluck('qualifier_id', R.values( R.prop('qualifiers', event ) ) );

    // excluded types of pass events (cross, throw in, keeper throw)
    // as per opta docs "Appendix 3 Useful Queries"
    // http://www.optasports.com/praxis/documentation/football-feed-appendices/f24-appendices.aspx
    if( R.gt( R.length( R.intersection( [2,107,123], qualifierIds) ), 0 )) {
        return context;
    }


    var startX = R.prop('x', event);
    var startY = R.prop('y', event);
    var endX = R.path(['qualifiers', 'pass-end-x', 'value'], event);
    var endY = R.path(['qualifiers', 'pass-end-y', 'value'], event);
    var xDiff = endX - startX;
    var outcome = R.prop('outcome', event);
    var side = context.teams[event.team.id].side.toLowerCase();
    var dir = 'Left to Right';

    var transformation = {
        particle: {
            data:{}
        }
    };
    transformation.particle.data[side] = {
        events:R.concat(R.__, [{
            start: {
                x: utils.translateX( dir, startX ),
                y: utils.translateY( dir, startY )
            },
            end: {
                x: utils.translateX( dir, endX ),
                y: utils.translateY( dir, endY )
            },
            type: 'pass',
            success: outcome === 1,
            side: side,
            static: false,
            player: {
                display: false
            }
        }]),         
        successful: R.when( R.equals(R.__, 1), R.always( R.add(1) ) )( outcome ),
        unsuccessful: R.when( R.equals(R.__, 0), R.always( R.add(1) ) )( outcome ),
        long: R.when( R.contains( R.__, qualifierIds), R.always( R.add(1) ) )(1),
        through: R.when( R.contains( R.__, qualifierIds), R.always( R.add(1) ) )(4),
        offensive: R.when( R.gt(R.__, 1), R.always( R.add(1) ) )(xDiff),
        defensive: R.when( R.lt(R.__, 0), R.always( R.add(1) ) )(xDiff),
        sideways: R.when( R.equals(R.__, 0), R.always( R.add(1) ) )(xDiff),
    };

    return R.evolve( transformation, context );
};
