var utils = require('./utils');

// cell dimensions as percentage of pitch,
// both should divide 100 cleanly into integers
var cell_width = 10;
var cell_height = 10;

function make_cells() {
    var cells = [];
    for (var x = 0; x < Math.ceil(100/cell_width) + 1; x++) {
	var column = [];
	for (var y = 0; y < Math.ceil(100/cell_height) + 1; y++) {
	    column.push(0);
	}
	cells.push(column);
    }
    return cells;
}

exports.initial = function (config) {
    return {
	particle: {
	    data: {
		home: {
		    num_events: 0,
		    counters: make_cells(),
		    team_percents: make_cells(),
		    total_percents: make_cells()
		},
		away: {
		    num_events: 0,
		    counters: make_cells(),
		    team_percents: make_cells(),
		    total_percents: make_cells()
		}
	    }
	},
        teams: config.teams
    };
};

function constrain(n, lower, upper) {
    return Math.min(Math.max(n, lower), upper);
}

exports.reducer = function (context, event) {
    var time = utils.getFormattedTime(event);
    context.particle.title = 'Average touch position (' + time + ')';
    
    if (event.type.name === 'Start') {
        var direction = event.qualifiers['direction-of-play'].value;
        context.teams[event.team.id].direction = direction;
    }
    else if ( event.type.name === 'End' ) {
        if ( event.period_id === 1 ) {
            context.particle.title = 'Average touch position (half time)';
        } else if ( event.period_id === 2 ) {
            context.particle.title = 'Average touch position (full time)';
        }
    }
    else if (event.type.name !== 'Team set up' &&
	     event.type.name !== 'Player on' &&
	     event.type.name !== 'Player Off' &&
	     event.type.name !== 'Card' &&
	     event.team && event.player && 'x' in event && 'y' in event) {

	var team = context.teams[event.team.id];
	var side = team.side.toLowerCase();
	var data = context.particle.data;

	// shift pitch co-ordinate by half a cell so that the edge
	// cells center on corners, sidelines etc.
	var pitch_x = Number(event.x) + cell_width/2;
	var pitch_y = Number(event.y) + cell_height/2;
	
	pitch_x = utils.absoluteX(
	    'Left to Right',
	    constrain(pitch_x, 0, 100 + cell_width)
	);
	pitch_y = utils.absoluteY(
	    'Left to Right',
	    constrain(pitch_y, 0, 100 + cell_height)
	);
	
	var cell_x = constrain(Math.round(pitch_x/cell_width) - 1, 0, 100);
	var cell_y = constrain(Math.round(pitch_y/cell_height) - 1, 0, 100);
	
	var counters = data[side].counters;
	var team_percents = data[side].team_percents;
	var total_percents = data[side].total_percents;
	
	data[side].num_events++;
	counters[cell_x][cell_y]++;

	var team_count = data[side].num_events;
	var total_count = data.home.num_events + data.away.num_events;
	
	for (var x = 0; x < Math.ceil(100/cell_width) + 1; x++) {
	    for (var y = 0; y < Math.ceil(100/cell_height) + 1; y++) {
		team_percents[x][y] = counters[x][y] / team_count * 100;
		total_percents[x][y] = counters[x][y] / total_count * 100;
	    }
	}
    }
    return context;
};
