var debug = false;
var R = require ( 'ramda' );
var utils = require ( './utils' );

var TIME_WINDOWS = [ 10, 15 ];

var log = function () {
    if ( debug ) {
        console.log.apply ( console, Array.prototype.slice.call ( arguments ) );
    }
};

var initTimeWindow = function ( teams ) {
    return {
        home: {
            seconds: 0,
            formatted_time: '00:00',
            fraction: 0.5,
            percent: '50%',
            percent_rounded: '50%'
        },
        away: {
            seconds: 0,
            formatted_time: '00:00',
            fraction: 0.5,
            percent: '50%',
            percent_rounded: '50%'
        },
        notInPlay: {
            name: "Not in play",
            seconds: 0,
            formatted_time: '00:00',
            fraction: 0.5
        }
    };
};

var hydrateParticle = function ( context, event ) {
    context.particle.data.total = recalc ( context.time.home, context.time.away, context.time.notInPlay );

    context.particle.data.total.timeWindow = [ [
        utils.getSemiFormattedTime ( R.assocPath ( [ 'min' ], 0, event ) ),
        utils.getSemiFormattedTime ( event )
    ].join ( ' - ' ), 'min' ].join ( ' ' );

    TIME_WINDOWS.forEach ( function ( minutes ) {
        var seconds = {
            home: context.time.home,
            away: context.time.away,
            notInPlay: context.time.notInPlay
        };

        for ( var i = 0; i < context.addStream.length; i++ ) {
            if ( context.addStream[i][0] > event.timestamp.unix - ( minutes * 60 ) ) {
                break;
            }
            seconds[context.addStream[i][1]] -= context.addStream[i][2];
        }

        context.particle.data.windows[minutes] = recalc ( seconds.home, seconds.away, seconds.notInPlay );

        if (event.min) {
            context.particle.data.windows[minutes].timeWindow = [ [
                utils.getSemiFormattedTime ( R.assocPath ( [ 'min' ], Math.max(event.min - minutes, 0), event ) ),
                utils.getSemiFormattedTime ( event )
            ].join ( ' - ' ), 'min' ].join ( ' ' );
        }
    } );

    return context;
};

var recalc = function ( homeSeconds, awaySeconds, notInPlaySeconds ) {
    var rc = {
        home: { seconds: homeSeconds },
        away: { seconds: awaySeconds },
        notInPlay: { seconds: notInPlaySeconds }
    };
    var total = homeSeconds + awaySeconds + notInPlaySeconds;
    var teamsTotal = homeSeconds + awaySeconds;

    ['home', 'away', 'notInPlay'].forEach(function(side) {
        rc[side].formatted_time = toMinSec(rc[side].seconds);
        rc[side].fraction = ( ( total === 0 ) ? 0.33 : ( rc[side].seconds / total ) );
        if (side !== 'notInPlay') {
            rc[side].percent = toPercentage ( ( teamsTotal === 0 ) ? 0.5 : rc[side].seconds/teamsTotal );
            rc[side].percent_rounded = toPercentage ( ( teamsTotal === 0 ) ? 0.5 : rc[side].seconds/teamsTotal, side );
        }
    });
    log ( rc );
    return rc;
};

var toPercentage = function ( fraction, side ) {
    var txFunc = {
        home: Math.floor,
        away: Math.ceil,
        notInPlay: Math.round
    }[side] || R.identity;

    return txFunc ( fraction * 100 ) + '%';
}

var toMinSec = function ( seconds ) {
    var pad0 = function ( n ) {
        return ((n<10)?'0':'') + n;
    }
        
    return [ pad0(Math.floor(seconds/60)), pad0(seconds%60) ].join ( ':' );
};

var max = function ( xs ) {
    return Math.max.apply(Math, xs);
};

var otherSide = function (side) {
    return side === 'home' ? 'away' : 'home';
};

exports.assemblerType = 'rbb-possession-windows';
exports.particleType = 'rbb-possession-windows';

exports.initial = function ( config ) {
    var homeTeamName = utils.chooseTeamName ( R.values ( config.teams ), 'Home' );
    var awayTeamName = utils.chooseTeamName ( R.values ( config.teams ), 'Away' );

    var context = {
        particle: {
            version: config.version,
            type: exports.particleType,
            assemblerType: exports.assemblerType,
            title: 'Possession: ' + homeTeamName + ' vs ' + awayTeamName,
	        data: {
                venue: config.venue,
                teams: {
                    home: {
                        score: 0,
                        rbbName: homeTeamName
                    },
                    away: {
                        score: 0,
                        rbbName: awayTeamName
                    }
                },
		        total: initTimeWindow ( config.teams ),
		        windows: TIME_WINDOWS.reduce ( function ( windows, interval ) {
		            windows[interval] = initTimeWindow ( config.teams );
		            return windows;
		        }, {} )
	        }
        },
        currentSide: 'notInPlay',
        startOfChain: null,
        time: {
            home: 0,
            away: 0,
            notInPlay: 0
        },
        lastThree: [],
        addStream: [],
        lastTimestamp: 0,
        teams: config.teams
    };
    return context;
};

exports.reducer = function (context, event) {
    var ignoreEvents = [
        'Good skill',
        'Keeper Sweeper',
        'Condition change',
        'Aerial',
        'Deleted event',
        'Offside provoked',
        'Card',
        'Penalty faced',
        'Collection End',
        'Player Off',
        'Player on',
        'Player retired',
        'Player returns',
        'Goalkeeper becomes player',
        'Player becomes goalkeeper',
        'Official change',
        'Player changed position',
        'Player changed Jersey number',
        'Collection End',
        'Temp_Goal',
        'Temp_Attempt',
        'Rescinded card',
        'Cross not claimed',
        'Shield ball opp',
        'Contentious referee decision',
        'Injury Time Announcement',
        'Chance missed',
        'Temp_Save',
        'Resume',
        '50/50',
        'Failed to Block',
        'Coach Setup',
        'Error',
        'Team set up'
    ];

    var breakEvents = [
        'Out',
        'Offside Pass',
        'Caught Offside',
        'Goal',
        'Foul',
        'Foul throw-in',
        'Corner Awarded',
        'Formation change',
        'Referee Drop Ball',
        'Collection End'
    ];

    log ( '==========' );
    log ( 'type', event.type.name );
    log ( 'timestamp', event.timestamp.unix );

	var side = context.teams[event.team.id].side.toLowerCase();
	var other = otherSide(side);

    var mapEvent = function ( event ) {
        return {
            time: event.timestamp ? event.timestamp.unix : event.time,
            side: event.side
        };
    };

    var addToTeam = function ( side, time ) {
        if ( context.startOfChain === null ) { return; }
        context.addStream.push ( [ context.startOfChain, side, time - context.startOfChain ] );
        context.time[side] += time - context.startOfChain;
        context.startOfChain = time;
    };

    //log ( 'last four before:' );
    //log ( context.lastThree );

    if ( R.contains ( event.type.name, ignoreEvents ) ) {
        log ( 'ignore event' );
        return context;
    }

    if ( event.period_id !== 1 && event.period_id !== 2 ) {
        log ( 'wrong period', event.period_id );
        return context;
    }

    if ( ! event.team || ! event.timestamp || ! event.timestamp.unix ) {
        log ( 'no team or no timestamp' );
        return context;
    }

    if ( event.timestamp.unix < context.lastTimestamp ) {
        log ( 'this event is out of order - ignore' );
        return context;
    }

    context.lastTimestamp = event.timestamp.unix;

    if ( event.type.name === 'Goal' ) {
        log ( 'update goal' );
        if ( R.has ( 'own-goal', event.qualifiers ) ) {
            context.particle.data.teams[other].score += 1;
        } else {
            context.particle.data.teams[side].score += 1;
        }
    }

    if ( event.type.name === 'Start' || event.type.name === 'End delay' ) {
        log ( 'start' );
        context.startOfChain = event.timestamp.unix;
        return context;
    }

    if ( event.type.name === 'End' || event.type.name === 'Start delay' ) {
        log ( 'end' );
        addToTeam ( context.currentSide, event.timestamp.unix );
        context.currentSide = 'notInPlay';
        context.startOfChain = null;
        return hydrateParticle ( context, event );
    }

    if ( context.startOfChain === null ) {
        log ( 'no startOfChain' );
        return context;
    }

    if ( R.contains ( event.type.name, breakEvents ) ) {
        log ( 'break' );
        addToTeam ( context.currentSide, event.timestamp.unix );
        context.currentSide = 'notInPlay'
        context.lastThree = [];
        return hydrateParticle ( context, event );
    }

    context.lastThree = R.map ( mapEvent, R.slice ( 0, 3, R.concat ( [ R.merge ( event, { side: side } ) ], context.lastThree ) ) );
    log ( context.lastThree );

    if ( context.currentSide === 'notInPlay' ) {
        log ( 'notInPlay' );
        addToTeam ( context.currentSide, context.lastThree[0].time );
        context.currentSide = side;
        return hydrateParticle ( context, event );
    }

    if ( side !== context.currentSide && context.lastThree.length === 3 && context.lastThree[0].side === context.lastThree[1].side && context.lastThree[1].side !== context.lastThree[2].side ) {
        log ( 'side switched' );
        addToTeam ( context.currentSide, context.lastThree[1].time );
        context.currentSide = side;
    }

    log ( context.currentSide, context.startOfChain, context.time );

    return hydrateParticle ( context, event );
};
