/*



IMPORTANT

This reducer is being deprecated in favor of 'rbb-shots-on-goal'
1. Better template
2. More accurate particle name




 */




var utils = require('./utils');
var R = require('ramda');

// scale mappings for goal mouth front elevation template
var GOAL_MOUTH_Y_MULTIPLIER = 18.2;
var GOAL_MOUTH_Y_MIN = 39.7;
var GOAL_MOUTH_Y_MAX = 60.3;
var GOAL_MOUTH_Z_MULTIPLIER = 2.35;
var GOAL_MOUTH_Z_MAX = 80;

var sortByGoal = R.sortBy( R.prop('is-goal') );

var assemblerType = 'rbb-shots-on-goal-pitch';

exports.particleType = 'rbb-goal-front-elevation';

exports.initial = function (config) {
    return {
        particle: {
            title: 'Shots',
            type: exports.particleType,
            assemblerType: assemblerType,
            version: config.version,
            uniqueId: [ assemblerType, 0 ].join ( '-' ),
            data: {
                side: config.side,
                events: [],
                'cumulative-shots': {
                    'goals': 0,
                    'match': 0,
                    'match-on-target': 0,
                    'first-half': 0,
                    'first-half-on-target': 0,
                    'second-half': 0,
                    'second-half-on-target': 0,
                    'proportion-in-second-half': 0,
                    'proportion-in-second-half-on-target': 0
                },
                'current-period': {
                    'id': -1
                }
            }
        },
	penalties: config.penalties,
	side: config.side,
        teams: config.teams,
        players: {},
        events: [],
        meta: {
            type: 'shots-on-goal',
            publishTags: [ 'reducer:rbb-shots-on-goal-pitch' ],
            generatedTimeStamp: null,
            eventTimeStamp: null           
        },
        gameTime: null,
    };
};

exports.reducer = function (context, event) {    
    utils.extendTeamData(context, event);
    utils.extendPlayerData(context, event);

    if (event.team) {
        var side = context.teams[event.team.id].side.toLowerCase();
    }
    if (event.type.name === 'Team set up' && side === context.side) {
        context.particle.title = context.teams[event.team.id].name + ' shots';
    }
    else if (event.type.name === 'Start' && side === context.side) {
        context.particle.title = context.teams[event.team.id].name + (
            context.penalties ? ' penalties' : ' shots'
        );
    }
    else if (((context.penalties && event.period_id === 5) || // penalities
              (!context.penalties && event.period_id !== 5)) && // not penalities
             utils.isShot(event) ) {

        var qs = event.qualifiers;
        var isOwnGoal = R.has('own-goal', event.qualifiers );

        // if the opposition scored an own goal, we'll keep it for the graphic's sake
        var isValid = ( (side === context.side && !isOwnGoal) || (side !== context.side && isOwnGoal) );
        if(!isValid) {
            return context;
        }

        var didReachGoalLine = qs.hasOwnProperty('goal-mouth-y-co-ordinate');
        var isBlocked = qs.hasOwnProperty('blocked-x-co-ordinate');



        var player = context.players[event.player.id];
        var jersey =  player ? player.jersey_number : '';
        var direction = context.teams[event.team.id].direction;

        var forceDirection = 'Left to right';
        
        /*
        Notes:

        "Goal Mouth" refers to the goal line, not just the goal area
         
        There are some shots that don't make it all the way to the goal line
        Some shots have a blocked location and a goal line location

        Appendix 12 on opta: http://www.optasports.com/praxis/documentation/football-feed-appendices/f24-appendices.aspx

        Goal mouth is the goal line from 0 to 100, going from right to left when looking at goal
        To work with templates, we need to have it from 0 to 100 left to right, but also scaled to match the scale of templates
        (see vars at top)
         */        

        var endCoords = {};
        var goalMouthCoords = {};
        if( didReachGoalLine ) {
            endCoords.x = 100;
            endCoords.y = event.qualifiers['goal-mouth-y-co-ordinate'].value;

            goalMouthCoords.y = utils.invert100AndMultiply( event.qualifiers['goal-mouth-y-co-ordinate'].value, GOAL_MOUTH_Y_MULTIPLIER);
            goalMouthCoords.z = utils.invert100AndMultiply( event.qualifiers['goal-mouth-z-co-ordinate'].value, GOAL_MOUTH_Z_MULTIPLIER);

        } else if( isBlocked ) {
            endCoords.x = event.qualifiers['blocked-x-co-ordinate'].value;
            endCoords.y = event.qualifiers['blocked-y-co-ordinate'].value;
        }

        var blockedCoords = {};
        if( isBlocked ) {
            blockedCoords.x = utils.translateX(direction, event.qualifiers['blocked-x-co-ordinate'].value);
            blockedCoords.y = utils.translateY(direction, event.qualifiers['blocked-y-co-ordinate'].value);
        }
	
        var qualifiers = utils.getEventQualifiersAsBooleans(event);
        var shotType = utils.qualifyShotType( event, qualifiers );
	
        var shotObj = {
            id: event.id,
            eid: event.event_id,
            type: 'shot ' + shotType,
            side: side,
            player: {
                jersey: jersey,
                display: true
            },
            start: {
                x: utils.translateX(direction, event.x),
                y: utils.translateY(direction, event.y)
            },
            end: {
                x: utils.translateX(direction, endCoords.x),
                y: utils.translateY(direction, endCoords.y)
            },
            mid: [blockedCoords],
            "is-goal": shotType === 'goal',
            "is-own-goal": isOwnGoal,
            "has-midpoints": isBlocked,
            "did-reach-goal-line": didReachGoalLine,
            "is-blocked": isBlocked,
            "goal-mouth": goalMouthCoords,
            "is-first-half": event.period_id === 1,
            "is-penalties": event.period_id === 5,
            "is-on-target": utils.isShotOnTarget(event),
            qualifiers: qualifiers

        };

        context.events.push(shotObj);

        // sorted so goals are last in the array, drawn on top for template
        context.particle.data.events = sortByGoal(context.events);

        var shots = context.particle.data['cumulative-shots'];
        // don't count the opposition's own goal as positive team stats for this team
        if( !isOwnGoal && !shotObj['is-penalties'] ) {
            // set up the shot stat totals for rules engine to examine
            shots.match++;
            if(shotObj['is-on-target']) {
                shots['match-on-target']++;
            }

            // note: second half includes extra time halves
            var bucket = (shotObj['is-first-half']) ? 'first-half' : 'second-half';
            shots[bucket]++;
            if(shotObj['is-on-target']) {
               shots[bucket + '-on-target']++; 
            }
        }

        context.particle.data['cumulative-shots'].goals = R.length( R.filter( R.propEq('is-goal', true), context.events ) );
        context.particle.data['cumulative-shots']['proportion-in-second-half'] = shots['second-half'] / shots['match'];
        context.particle.data['cumulative-shots']['proportion-in-second-half-on-target'] = shots['second-half-on-target'] / shots['match-on-target'];
        
        context.particle.uniqueId = [ assemblerType, event.id ].join ( '-' );
    }

    context.gameTime = utils.createGameTimeObject(event, context.gameTime);
    // opta recorded time
    context.meta.eventTimeStamp = utils.getOptaEventTimeStamp(event);
    // set up the current period indicators for rules engine
    // this represents the latest game event, not necessarily an event that this reducer is interested in
    context.particle.data['current-period'] = utils.setTimePeriodForRules(event, context);
    return context;
};
