var utils = require('./utils');

exports.particleType = "rbb-single-pitch";

exports.initial = function (config) {
    return {
        sides: config.sides,
        players: {},
        teams: {},
        data: {
            showDirectionOfAttack: true,
            "is-average-touch": true,
            events: []
        },
        meta: {
            type: 'average-touch',
            publishTags: [ 'reducer:rbb-team-average-touch-position' ],
            generatedTimeStamp: null,            
            eventTimeStamp: null            
        },
        gameTime: null
    };
};

exports.reducer = function (context, event, config) {
    if (event.team) {
        var side = config.sides[event.team.id].toLowerCase();
    }
    if (side && side != config.side) {
        return context;
    }
    if (event.type.name === 'Team set up') {
        var team = {
            id: event.team.id,
            name: event.team.name
        };
        context.title = event.team.name + ' average touch positions';
        context.teams[team.id] = team;
        team.involved = event.qualifiers['involved'].value.split(', ');
        team.jersey_numbers = event.qualifiers['jersey-number'].value.split(', ');
        team.player_positions = event.qualifiers['player-position'].value.split(', ');
        team.involved.forEach(function (id, i) {
            context.players[id] = {
                jersey_number: team.jersey_numbers[i],
                position: team.player_positions[i],
                total: {x: 0, y: 0},
                average: {},
                count: 0,
                team: event.team.id
            };
        });
    }
    else if (event.type.name === 'Start') {
        var direction = event.qualifiers['direction-of-play'].value;
        context.teams[event.team.id].direction = direction;
    }
    else if (event.player && 'x' in event && 'y' in event) {
        var player = context.players[event.player.id];
        player.count++;
        player.total.x += Number(event.x);
        player.total.y += Number(event.y);
        player.average.x = player.total.x / player.count;
        player.average.y = player.total.y / player.count;

        context.data.events = [];
        for (var player_id in context.players) {
            team = context.teams[player.team];
            player = context.players[player_id];
            if (player.count > 0) {
                context.data.events.push({
                    type: 'dribble',
                    side: context.sides[player.team].toLowerCase(),
                    qualifiers: {},
                    player: {
                        jersey: player.jersey_number,
                        substitute: player.position === '5',
                        display: true
                    },
                    start: {
                        x: utils.translateX('Left to Right', player.average.x),
                        y: utils.translateY('Left to Right', player.average.y)
                    },
                    end: {
                        x: utils.translateX('Left to Right', player.average.x),
                        y: utils.translateY('Left to Right', player.average.y)
                    }
                });
            }
        }

        context.gameTime = utils.createGameTimeObject(event, context.gameTime);

        // opta recorded time
        context.meta.eventTimeStamp = utils.getOptaEventTimeStamp(event);

    }
    return context;
};
