var R = require ( 'ramda' );
var utils = require ( './utils' );

exports.particleType = 'rbb-touchmap';
exports.assemblerType = 'rbb-touchmap';

var initThirds = function () {
    return {
        defence: {
            total: 0,
            percentage: 33
        },
        midfield: {
            total: 0,
            percentage: 33
        },
        attack: {
            total: 0,
            percentage: 33
        }
    };
};

var isTouchInBox = function(x,y) {
    return x > 83 && x < 100 && y < 78.9 && y > 21;
};

exports.initial = function ( config ) {

    return {
        particle: {
            version: config.version,
            type: exports.particleType,
            assemblerType: exports.assemblerType,
            title: utils.chooseTeamName ( R.values ( config.teams ), 'Home' ) + ' vs ' + utils.chooseTeamName ( R.values ( config.teams ), 'Away' ),
            data: {
                home: {
                    name: utils.chooseTeamName ( R.values ( config.teams ), 'Home' ),
                    events: [],
                    total: 0,
                    inBox: 0,
                    thirds: initThirds ()
                },
                away: {
                    name: utils.chooseTeamName ( R.values ( config.teams ), 'Away' ),
                    events: [],
                    total: 0,
                    inBox: 0,
                    thirds: initThirds ()
                }
            }
        },
        teams: config.teams
    };
};

exports.reducer = function ( context, event ) {
    if( ! utils.isTouchEvent ( event ) ) {
        return context;
    }

    var side = context.teams[event.team.id].side.toLowerCase();
    var dir = 'Left to Right';
    var x = R.prop ('x', event );
    var y = R.prop ('y', event );
    var isInBox = isTouchInBox(x,y);
    var transformation = {
        particle: {
            data: R.fromPairs ( [ R.pair ( side, {
                events: R.concat ( R.__, [ {
                    start: {
                        x: utils.translateX ( dir, x ),
                        y: utils.translateY ( dir, y )
                    },
                    inBox: isTouchInBox(x,y),
                    type: 'position',
                    side: side,
                    static: true
                } ] ),
                total: R.inc,
                inBox: isInBox ? R.inc : R.identity,
                thirds: utils.transformTouchThirds ( event )
            } ) ] )
        }
    };
       
    return R.evolve( transformation, context );
};
