var ParticleAssembler = require('../index');
var request = require('request');


function startAssembler(competitionId, seasonId, fixtureId, callback) {
    var api_uri = 'http://opta-api-staging.eip.telegraph.co.uk/events/' + competitionId + '/' + seasonId + '/' + fixtureId;
    var api_uri_publishes = 'http://opta-api-staging.eip.telegraph.co.uk/publishes/' + competitionId + '/' + seasonId + '/' + fixtureId;
    var socketio_uri = 'http://wss-staging.eip.telegraph.co.uk/opta-events';

    var subscriber = new ParticleAssembler.EventSubscriber(api_uri, socketio_uri);
    var publishSubscriber = new ParticleAssembler.EventSubscriber(api_uri_publishes, socketio_uri);
    var cache = new ParticleAssembler.MemoryCache();

    var checkpointSize = 250;

    var f9 = 'https://opta-api-staging.eip.telegraph.co.uk/feeds/F9/' + competitionId + '/' + seasonId + '/' + fixtureId;
    request.get(f9, function (err, response) {
        if (err) {
            throw err;
        }
        var f9data = JSON.parse(response.body);
        var teams = f9data.competition.fixtures[0].teams;
        var teamsForPlayers = f9data.competition.teams;

        var globalConfig = {
            version: '0.1.2',
            teams: {},
            sides: {}
        };

        globalConfig.sides[teams[0].id] = teams[0].side;
        globalConfig.sides[teams[1].id] = teams[1].side;

        globalConfig.teams[teams[0].id] = {
            id: teams[0].id,
            name: teams[0].name,
            side: teams[0].side,
            players: teamsForPlayers.find ( function ( team ) {
                return team.id === teams[0].id;
            } ).players
        };

        globalConfig.teams[teams[1].id] = {
            id: teams[1].id,
            name: teams[1].name,
            side: teams[1].side,
            players: teamsForPlayers.find ( function ( team ) {
                return team.id === teams[1].id;
            } ).players
        };

        var assembler = new ParticleAssembler.ParticleAssembler(
            subscriber,
            publishSubscriber,
            cache,
            globalConfig,
            checkpointSize
        );

        var particle = assembler.add({
            type: 'rbb-goals-and-passes',
            config: {}
        });

        var total_build_time = 0;
        var total_updates = 0;

        particle.on('update', function (context, build_time) {
            //console.log('================');
            //console.log(JSON.stringify(context, null, 4));
            //total_build_time += build_time;
            //total_updates++;
            //console.log('--- ' + build_time.toFixed(3) + 'ms (' + (total_build_time / total_updates).toFixed(3) + 'ms average) ---');
        });
        callback(null, assembler);
    });
}


var competitionId = 8,
    seasonId = 2016,
    fixtureId = 855173;

var counter = 100;
function next(err, assembler) {
    if (err) {
        throw err;
    }
    counter--;
    if (counter) {
        setTimeout(function () {
            var mem = process.memoryUsage();
            console.log(mem.heapUsed + ' / ' + mem.heapTotal);
            assembler.destroy();
            startAssembler(competitionId, seasonId, fixtureId, next);
        }, 1000);
    }
}
startAssembler(competitionId, seasonId, fixtureId, next);
