var ParticleAssembler = require('../index');
var request = require('request');
var reducerConfig = require ( '../lib/build-reducer-config.js' )( '0.1.6' );
var async = require ( 'async' );

var competitionId = process.argv[2] || 8, seasonId = process.argv[3] || 2016, fixtureId = process.argv[4] || 855173;

var api_uri = `http://opta-api-staging.eip.telegraph.co.uk/events/${competitionId}/${seasonId}/${fixtureId}`;
var api_uri_publishes = `http://opta-api-staging.eip.telegraph.co.uk/publishEvents/${competitionId}/${seasonId}/${fixtureId}`;
var socketio_uri = 'http://wss-staging.eip.telegraph.co.uk/opta-events';

var subscriber = new ParticleAssembler.EventSubscriber(api_uri, socketio_uri);
var publishSubscriber = new ParticleAssembler.EventSubscriber(api_uri_publishes, socketio_uri);
var cache = new ParticleAssembler.MemoryCache();

var checkpointSize = 250;

var f9 = `https://opta-api-staging.eip.telegraph.co.uk/feeds/F9/${competitionId}/${seasonId}/${fixtureId}`;
var f40 = `https://opta-api-staging.eip.telegraph.co.uk/feeds/F40/${competitionId}/${seasonId}`;

async.parallel ( [
    function ( callback ) { request.get ( { url: f9, json: true }, callback ); },
    function ( callback ) { request.get ( { url: f40, json: true }, callback ); }
], function ( err, responses ) {
    if (err) {
        throw err;
    }

    if ( responses.find ( function ( response ) {
        return response[0].statusCode !== 200;
    } ) ) {
        throw new Error ( 'HTTP request fails' );
    }

    var globalConfig = reducerConfig ( {
        F9: responses[0][0].body,
        F40: responses[1][0].body
    } );

    var assembler = new ParticleAssembler.ParticleAssembler(
        subscriber,
        publishSubscriber,
        cache,
        globalConfig,
        checkpointSize
    );

    var particle = assembler.add({
        type: 'rbb-landscape',
        config: {}
    });

    var total_build_time = 0;
    var total_updates = 0;

    particle.on('update', function (context, build_time) {
        console.log('================');
        console.log(JSON.stringify(context, null, 4));
        total_build_time += build_time;
        total_updates++;
        console.log('--- ' + build_time.toFixed(3) + 'ms (' + (total_build_time / total_updates).toFixed(3) + 'ms average) ---');
    });
} );
