var EventEmitter = require('events').EventEmitter;
var inherits = require('inherits');
var url = require('url');
var request = require(process.env.BROWSER ? 'xhr': 'request');
var io = require('socket.io-client');

var BATCH_SIZE = 1000;


function EventSubscriber(api_uri, socketio_uri, since, id_token, sig) {
    EventEmitter.call(this);
    this.setMaxListeners(100);
    this.uri = api_uri;
    this.head = since || 0;
    this.id_token = id_token || null;
    this.sig = sig || null;
    this.remote_head = null;
    this.channel = null;

    var self = this;
    info(this.uri, self.id_token, self.sig, function (err, data) {
        if (err) {
            self.emit('error', err);
            return;
        }
        //console.log(['info', data]);
        self.remote_head = data.head;
        if ( socketio_uri !== null ) {
            self.socket = io(socketio_uri);
            self.socket.on('error', function (err) {
                //console.error(err);
                self.emit('error', err);
            });
            self.socket.on('joined', function (name) {
                //console.log('Joined channel ' + name);
            });
            self.socket.on('event', function (event) {
                if (event.channel !== self.channel_name) {
                    //console.log(['channel event', event, 'ignoring event for different channel']);
                }
                else if (self.ignore_channel){
                    //console.log(['channel event', event, 'ignored during catchup']);
                    return;
                }
                else if (event.sequenceId === self.head + 1) {
                    //console.log(['channel event', event, 'next in sequence']);
                    self.remote_head++;
                    self.pushEvent(event);
                }
                else if (event.sequenceId > self.head + 1) {
                    //console.log(['channel event', event, 'future event']);
                    self._catchup();
                }
                else {
                    //console.log(['channel event', event, 'past event (ignored)']);
                    // ignore past event
                }
            });
            self.socket.on('connect', function () {
                //console.log('socket.io connected');
                self.subscribe(data.channelName);
                if (!self.caughtUp()) {
                    self._catchup();
                }
            });
        } else {
            if (!self.caughtUp()) {
                self._catchup();
            }
        }
    });
}
inherits(EventSubscriber, EventEmitter);

EventSubscriber.prototype.disconnect = function () {
    if (this.socket) {
        this.socket.disconnect();
    }
    this.removeAllListeners();
};

EventSubscriber.prototype.subscribe = function (channel) {
    this.channel_name = channel;
    //console.log('Joining channel ' + this.channel_name);
    this.socket.emit('join', this.channel_name);
};

EventSubscriber.prototype.pushEvent = function (event) {
    this.head = event.sequenceId;
    this.emit('event', event);
};

EventSubscriber.prototype._catchup = function () {
    //console.log(['_catchup']);
    var self = this;
    this.ignore_channel = true;
    batch(self.uri, self.head, BATCH_SIZE, self.id_token, self.sig, function (err, data) {
        //console.log(['got batch', data]);
        self.remote_head = data.head;
        self.ignore_channel = false;
        if (err) {
            self.emit('error', err);
            return;
        }
        data.events.forEach(function (event) {
            if (event.sequenceId === self.head + 1) {
                self.pushEvent(event);
            }
        });
        if (!self.caughtUp()) {
            // more data, request another batch
            self._catchup();
        }
    });
};

EventSubscriber.prototype.caughtUp = function () {
    //console.log(['caughtUp', this.head, this.remote_head]);
    return this.remote_head !== null && this.head === this.remote_head;
};

function info(uri, id_token, sig, callback) {
    batch(uri, 0, 0, id_token, sig, callback);
}

function batch(uri, start, count, id_token, sig, callback) {
    //console.log(['EventSubscriber batch', start, count]);
    uri = url.parse(uri, true);
    if (start !== null) {
        uri.query.start = start;
    }
    if (count !== null) {
        uri.query.count = count;
    }
    if (id_token !== null) {
        uri.query.id_token = id_token;
    } else {
        if (sig !== null) {
            uri.query.sig = sig;
        }
    }
    request.get(url.format(uri), function (err, response) {
        if (err) {
            return callback(err);
        }
        return callback(null, JSON.parse(response.body));
    });
}

module.exports = EventSubscriber;
