const H = require ( 'highland' );
const R = require ( 'ramda' );
const cache = require ( '../index.js' ).MemoryCache;
const subscriber = require ( '../index.js' ).EventSubscriber;

const S = new subscriber ( [
    'http://opta-api-staging.eip.telegraph.co.uk',
    'events',
    process.argv[2],
    process.argv[3],
    process.argv[4]
].join ( '/' ), null );

const C = new cache ();

const typeSeq = R.slice ( 5, process.argv.length, process.argv );

H ( ( push, next ) => {
    const checkTypeSeq = ( range ) => {
        return R.reduce ( ( matched, type ) => {
            return matched && type[0].data.type.name === type[1];
        }, true, R.zip ( R.slice ( range.length - typeSeq.length, range.length, range ), typeSeq ) );
    };

    S.on ( 'event', ( event ) => {
        const data = event.event;

        switch ( data.type ) {
            case 'append':
                C.append ( data.hash, data.event );
                break;
            case 'insertBefore':
                C.insertBefore ( data.before, data.hash, data.event );
                break;
            case 'replace':
                C.replace ( data.old, data.hash );
                break;
            case 'remove':
                C.remove ( data.hash );
                break;
        }
    } );

    C.on ( 'update', () => {
        if ( checkTypeSeq ( C.range ( 0, C.size () ) ) ) {
            push ( null, C.range ( 0, C.size () ) );
            next ( H ( [] ) );
        }
    } );

    C.on ( 'reset', ( index ) => {
        if ( checkTypeSeq ( C.range ( 0, C.size () ) ) ) {
            push ( null, C.range ( 0, C.size () ) );
            next ( H ( [] ) );
        }
    } );
} )
    .sequence ()
    .errors ( R.unary ( console.error ) )
    .each ( R.unary ( R.compose ( console.log, R.partialRight ( JSON.stringify, [ null, 2 ] ) ) ) );
