const H = require ( 'highland' );
const R = require ( 'ramda' );
const request = require ( 'request' );
const assembler = require ( '../index.js' ).ParticleAssembler;
const cache = require ( '../index.js' ).MemoryCache;
const subscriber = require ( '../index.js' ).EventSubscriber;
const reducerConfig = require ( '../lib/build-reducer-config.js' )( '0.2.0' );
const auth = require ( 'google-back-end-auth' );

const parseResponse = H.wrapCallback ( ( response, callback ) => {
    if ( response.statusCode !== 200 ) {
        return callback ( [ 'HTTP', 'code', response.statusCode, 'returned:', response.body ].join ( ' ' ) );
    }

    try {
        callback ( null, JSON.parse ( response.body ) );
    } catch ( error ) {
        callback ( error.message );
    }
} );

/*const googleConfig = {
    client_id: '932020179419-jj05sbi05en9f1naog1sng63o2640un5',
    client_secret: 'UnG4nmAZohgO8vOyQEMIkYQG'
};*/
const googleConfig = {
    client_id: '932020179419-7kubp7v2egbsa3nte7rtfvl0t7cni6c0',
    client_secret: 'jKalIJe_g4yb-kFoG4C4LZwy'
};

//const optaApiUrl = 'http://localhost:5000';
const optaApiUrl = 'http://opta-api-staging.eip.telegraph.co.uk';

H ( [
    H.wrapCallback ( request )( auth.signRequest ( googleConfig, {
        url: [ optaApiUrl, 'feeds', 'F9', process.argv[2], process.argv[3], process.argv[4] ].join ( '/' )
    } ) )
        .flatMap ( parseResponse ),
    H.wrapCallback ( request )( auth.signRequest ( googleConfig, {
        url: [ optaApiUrl, 'feeds', 'F40', process.argv[2], process.argv[3] ].join ( '/' )
    } ) )
        .flatMap ( parseResponse )
] )
    .parallel ( 2 )
    .collect ()
    .map ( R.compose ( R.fromPairs, R.zip ( [ 'F9', 'F40' ] ) ) )
    .map ( reducerConfig )
    .map ( ( globalConfig ) => {
        const eventsUrl = [ optaApiUrl, 'events', process.argv[2], process.argv[3], process.argv[4] ].join ( '/' );
        //const publishEventsUrl = [ optaApiUrl, 'publishEvents', process.argv[2], process.argv[3], process.argv[4] ].join ( '/' );
        const eventsSig = auth.sign ( googleConfig.client_id, googleConfig.client_secret, eventsUrl, 'GET' );
        //const publishEventsSig = auth.sign ( googleConfig.client_id, googleConfig.client_secret, publishEventsUrl, 'GET' );

        return new assembler (
            new subscriber ( eventsUrl, null, null, null, eventsSig ),
            null,
            //new subscriber ( publishEventsUrl, null, null, null, publishEventsSig ),
            new cache (),
            globalConfig
        );
    } )
    .map ( ( assembler ) => {
        return assembler.add ( { type: process.argv[5] || 'rbb-landscape', config: {} } );
    } )
    .flatMap ( ( particle ) => {
        return H ( ( push, next ) => {
            var index = 0;

            const pushParticle = ( particle ) => {
                setTimeout ( () => {
                    push ( null, JSON.stringify ( particle, null, 4 ) );
                }, 0 );
            };

            particle.on ( 'update', ( part ) => {
                if ( process.argv[7] ) {
                    const partPath = process.argv[6].split ( '.' );

                    particle.on ( 'update', pushParticle );
                    particle.pauseAt ( partPath, process.argv[7] );
                } else if ( process.argv[6] === 'last' ) {
                    pushParticle ( part );
                } else if ( typeof process.argv[6] !== 'undefined' ) {
                    index = parseInt ( process.argv[6], 10 );

                    if ( particle.index === index ) {
                        pushParticle ( part );
                    } else {
                        particle.pause ( index );
                    }
                } else {
                    if ( particle.index === index ) {
                        pushParticle ( part );
                        setTimeout ( () => {
                            index++;
                            particle.pause ( index );
                        }, 0 );
                    } else {
                        particle.pause ( index );
                    }
                }
            } );
        } );
    } )
    .errors ( R.unary ( console.error ) )
    .each ( R.unary ( console.log ) );
