const H = require ( 'highland' );
const R = require ( 'ramda' );
const request = require ( 'request' );

const callApi = url => {
    return H.wrapCallback ( request )( {
        url: url,
        json: true
    } )
        .flatMap ( H.wrapCallback ( ( response, callback ) => {
            if ( response.statusCode !== 200 ) {
                return callback ( `HTTP code ${response.statusCode} returned: ${JSON.stringify(response.body)}` );
            }

            return callback ( null, response.body );
        } ) );
};

const apiUrl = 'http://opta-api-staging.eip.telegraph.co.uk';
const competitionId = process.argv[2] || 8, seasonId = process.argv[3] || 2016;

callApi ( `${apiUrl}/feeds/F1/${competitionId}/${seasonId}` )
    .pluck ( 'competition' )
    .pluck ( 'fixtures' )
    .sequence ()
    .filter ( R.compose ( R.equals ( 'FullTime' ), R.prop ( 'period' ) ) )
    .map ( fixture => {
        return [ 'printf', '"%s, %s, %s, "', `"${fixture.id}"`, `"${fixture.teams[0].name}"`, `"${fixture.teams[1].name}";`, 'node', 'test/particleGenerator.js', competitionId, seasonId, fixture.id, 'rbb-landscape', 'last' ].join ( ' ' );
    } )
    .errors ( R.unary ( console.error ) )
    .each ( R.unary ( console.log ) );
